# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

V_5TTGEN_METADATA = Metadata(
    id="213c8cb30d8d1a68c488abbf08a42cf6809dc622.boutiques",
    name="5ttgen",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_V5ttgenFreesurferParamsDictNoTag = typing.TypedDict('_V5ttgenFreesurferParamsDictNoTag', {
    "input": InputPathType,
    "output": str,
    "lut": typing.NotRequired[InputPathType | None],
})
V5ttgenFreesurferParamsDictTagged = typing.TypedDict('V5ttgenFreesurferParamsDictTagged', {
    "@type": typing.Literal["freesurfer"],
    "input": InputPathType,
    "output": str,
    "lut": typing.NotRequired[InputPathType | None],
})
V5ttgenFreesurferParamsDict = _V5ttgenFreesurferParamsDictNoTag | V5ttgenFreesurferParamsDictTagged


_V5ttgenFslParamsDictNoTag = typing.TypedDict('_V5ttgenFslParamsDictNoTag', {
    "input": InputPathType,
    "output": str,
    "t2": typing.NotRequired[InputPathType | None],
    "mask": typing.NotRequired[InputPathType | None],
    "premasked": bool,
})
V5ttgenFslParamsDictTagged = typing.TypedDict('V5ttgenFslParamsDictTagged', {
    "@type": typing.Literal["fsl"],
    "input": InputPathType,
    "output": str,
    "t2": typing.NotRequired[InputPathType | None],
    "mask": typing.NotRequired[InputPathType | None],
    "premasked": bool,
})
V5ttgenFslParamsDict = _V5ttgenFslParamsDictNoTag | V5ttgenFslParamsDictTagged


_V5ttgenGifParamsDictNoTag = typing.TypedDict('_V5ttgenGifParamsDictNoTag', {
    "input": InputPathType,
    "output": str,
})
V5ttgenGifParamsDictTagged = typing.TypedDict('V5ttgenGifParamsDictTagged', {
    "@type": typing.Literal["gif"],
    "input": InputPathType,
    "output": str,
})
V5ttgenGifParamsDict = _V5ttgenGifParamsDictNoTag | V5ttgenGifParamsDictTagged


_V5ttgenHsvsParamsDictNoTag = typing.TypedDict('_V5ttgenHsvsParamsDictNoTag', {
    "input": InputPathType,
    "output": str,
    "template": typing.NotRequired[InputPathType | None],
    "hippocampi": typing.NotRequired[typing.Literal["subfields", "first", "aseg"] | None],
    "thalami": typing.NotRequired[typing.Literal["nuclei", "first", "aseg"] | None],
    "white_stem": bool,
})
V5ttgenHsvsParamsDictTagged = typing.TypedDict('V5ttgenHsvsParamsDictTagged', {
    "@type": typing.Literal["hsvs"],
    "input": InputPathType,
    "output": str,
    "template": typing.NotRequired[InputPathType | None],
    "hippocampi": typing.NotRequired[typing.Literal["subfields", "first", "aseg"] | None],
    "thalami": typing.NotRequired[typing.Literal["nuclei", "first", "aseg"] | None],
    "white_stem": bool,
})
V5ttgenHsvsParamsDict = _V5ttgenHsvsParamsDictNoTag | V5ttgenHsvsParamsDictTagged


_V5ttgenConfigParamsDictNoTag = typing.TypedDict('_V5ttgenConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
V5ttgenConfigParamsDictTagged = typing.TypedDict('V5ttgenConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
V5ttgenConfigParamsDict = _V5ttgenConfigParamsDictNoTag | V5ttgenConfigParamsDictTagged


_V5ttgenParamsDictNoTag = typing.TypedDict('_V5ttgenParamsDictNoTag', {
    "algorithm": typing.Union[V5ttgenFreesurferParamsDictTagged, V5ttgenFslParamsDictTagged, V5ttgenGifParamsDictTagged, V5ttgenHsvsParamsDictTagged],
    "nocrop": bool,
    "sgm_amyg_hipp": bool,
    "nocleanup": bool,
    "scratch": typing.NotRequired[str | None],
    "continue": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[V5ttgenConfigParamsDict] | None],
    "help": bool,
    "version": bool,
})
V5ttgenParamsDictTagged = typing.TypedDict('V5ttgenParamsDictTagged', {
    "@type": typing.Literal["mrtrix/5ttgen"],
    "algorithm": typing.Union[V5ttgenFreesurferParamsDictTagged, V5ttgenFslParamsDictTagged, V5ttgenGifParamsDictTagged, V5ttgenHsvsParamsDictTagged],
    "nocrop": bool,
    "sgm_amyg_hipp": bool,
    "nocleanup": bool,
    "scratch": typing.NotRequired[str | None],
    "continue": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[V5ttgenConfigParamsDict] | None],
    "help": bool,
    "version": bool,
})
V5ttgenParamsDict = _V5ttgenParamsDictNoTag | V5ttgenParamsDictTagged


def v_5ttgen_algorithm_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "freesurfer": v_5ttgen_freesurfer_cargs,
        "fsl": v_5ttgen_fsl_cargs,
        "gif": v_5ttgen_gif_cargs,
        "hsvs": v_5ttgen_hsvs_cargs,
    }.get(t)


def v_5ttgen_algorithm_outputs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "freesurfer": v_5ttgen_freesurfer_outputs,
        "fsl": v_5ttgen_fsl_outputs,
        "gif": v_5ttgen_gif_outputs,
        "hsvs": v_5ttgen_hsvs_outputs,
    }.get(t)


def v_5ttgen_algorithm_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "freesurfer": v_5ttgen_freesurfer_validate,
        "fsl": v_5ttgen_fsl_validate,
        "gif": v_5ttgen_gif_validate,
        "hsvs": v_5ttgen_hsvs_validate,
    }.get(t)


class V5ttgenFreesurferOutputs(typing.NamedTuple):
    """
    Output object returned when calling `V5ttgenFreesurferParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """The output 5TT image"""


def v_5ttgen_freesurfer(
    input_: InputPathType,
    output: str,
    lut: InputPathType | None = None,
) -> V5ttgenFreesurferParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: The input FreeSurfer parcellation image (any image containing\
            'aseg' in its name).
        output: The output 5TT image.
        lut: Manually provide path to the lookup table on which the input\
            parcellation image is based (e.g. FreeSurferColorLUT.txt).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "freesurfer",
        "input": input_,
        "output": output,
    }
    if lut is not None:
        params["lut"] = lut
    return params


def v_5ttgen_freesurfer_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `V5ttgenFreesurferParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')
    if params.get("lut", None) is not None:
        if not isinstance(params["lut"], (pathlib.Path, str)):
            raise StyxValidationError(f'`lut` has the wrong type: Received `{type(params.get("lut", None))}` expected `InputPathType | None`')


def v_5ttgen_freesurfer_cargs(
    params: V5ttgenFreesurferParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("freesurfer")
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("output", None))
    if params.get("lut", None) is not None:
        cargs.extend([
            "-lut",
            execution.input_file(params.get("lut", None))
        ])
    return cargs


def v_5ttgen_freesurfer_outputs(
    params: V5ttgenFreesurferParamsDict,
    execution: Execution,
) -> V5ttgenFreesurferOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = V5ttgenFreesurferOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


class V5ttgenFslOutputs(typing.NamedTuple):
    """
    Output object returned when calling `V5ttgenFslParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """The output 5TT image"""


def v_5ttgen_fsl(
    input_: InputPathType,
    output: str,
    t2: InputPathType | None = None,
    mask: InputPathType | None = None,
    premasked: bool = False,
) -> V5ttgenFslParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: The input T1-weighted image.
        output: The output 5TT image.
        t2: Provide a T2-weighted image in addition to the default T1-weighted\
            image; this will be used as a second input to FSL FAST.
        mask: Manually provide a brain mask, rather than deriving one in the\
            script.
        premasked: Indicate that brain masking has already been applied to the\
            input image.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "fsl",
        "input": input_,
        "output": output,
        "premasked": premasked,
    }
    if t2 is not None:
        params["t2"] = t2
    if mask is not None:
        params["mask"] = mask
    return params


def v_5ttgen_fsl_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `V5ttgenFslParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')
    if params.get("t2", None) is not None:
        if not isinstance(params["t2"], (pathlib.Path, str)):
            raise StyxValidationError(f'`t2` has the wrong type: Received `{type(params.get("t2", None))}` expected `InputPathType | None`')
    if params.get("mask", None) is not None:
        if not isinstance(params["mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType | None`')
    if params.get("premasked", False) is None:
        raise StyxValidationError("`premasked` must not be None")
    if not isinstance(params["premasked"], bool):
        raise StyxValidationError(f'`premasked` has the wrong type: Received `{type(params.get("premasked", False))}` expected `bool`')


def v_5ttgen_fsl_cargs(
    params: V5ttgenFslParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("fsl")
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("output", None))
    if params.get("t2", None) is not None:
        cargs.extend([
            "-t2",
            execution.input_file(params.get("t2", None))
        ])
    if params.get("mask", None) is not None:
        cargs.extend([
            "-mask",
            execution.input_file(params.get("mask", None))
        ])
    if params.get("premasked", False):
        cargs.append("-premasked")
    return cargs


def v_5ttgen_fsl_outputs(
    params: V5ttgenFslParamsDict,
    execution: Execution,
) -> V5ttgenFslOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = V5ttgenFslOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


class V5ttgenGifOutputs(typing.NamedTuple):
    """
    Output object returned when calling `V5ttgenGifParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """The output 5TT image"""


def v_5ttgen_gif(
    input_: InputPathType,
    output: str,
) -> V5ttgenGifParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: The input Geodesic Information Flow (GIF) segmentation image.
        output: The output 5TT image.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "gif",
        "input": input_,
        "output": output,
    }
    return params


def v_5ttgen_gif_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `V5ttgenGifParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def v_5ttgen_gif_cargs(
    params: V5ttgenGifParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("gif")
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("output", None))
    return cargs


def v_5ttgen_gif_outputs(
    params: V5ttgenGifParamsDict,
    execution: Execution,
) -> V5ttgenGifOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = V5ttgenGifOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


class V5ttgenHsvsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `V5ttgenHsvsParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """The output 5TT image"""


def v_5ttgen_hsvs(
    input_: InputPathType,
    output: str,
    template: InputPathType | None = None,
    hippocampi: typing.Literal["subfields", "first", "aseg"] | None = None,
    thalami: typing.Literal["nuclei", "first", "aseg"] | None = None,
    white_stem: bool = False,
) -> V5ttgenHsvsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: The input FreeSurfer subject directory.
        output: The output 5TT image.
        template: Provide an image that will form the template for the\
            generated 5TT image.
        hippocampi: Select method to be used for hippocampi (& amygdalae)\
            segmentation; options are: subfields,first,aseg.
        thalami: Select method to be used for thalamic segmentation; options\
            are: nuclei,first,aseg.
        white_stem: Classify the brainstem as white matter.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "hsvs",
        "input": input_,
        "output": output,
        "white_stem": white_stem,
    }
    if template is not None:
        params["template"] = template
    if hippocampi is not None:
        params["hippocampi"] = hippocampi
    if thalami is not None:
        params["thalami"] = thalami
    return params


def v_5ttgen_hsvs_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `V5ttgenHsvsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')
    if params.get("template", None) is not None:
        if not isinstance(params["template"], (pathlib.Path, str)):
            raise StyxValidationError(f'`template` has the wrong type: Received `{type(params.get("template", None))}` expected `InputPathType | None`')
    if params.get("hippocampi", None) is not None:
        if not isinstance(params["hippocampi"], str):
            raise StyxValidationError(f'`hippocampi` has the wrong type: Received `{type(params.get("hippocampi", None))}` expected `typing.Literal["subfields", "first", "aseg"] | None`')
        if params["hippocampi"] not in ["subfields", "first", "aseg"]:
            raise StyxValidationError("Parameter `hippocampi` must be one of [\"subfields\", \"first\", \"aseg\"]")
    if params.get("thalami", None) is not None:
        if not isinstance(params["thalami"], str):
            raise StyxValidationError(f'`thalami` has the wrong type: Received `{type(params.get("thalami", None))}` expected `typing.Literal["nuclei", "first", "aseg"] | None`')
        if params["thalami"] not in ["nuclei", "first", "aseg"]:
            raise StyxValidationError("Parameter `thalami` must be one of [\"nuclei\", \"first\", \"aseg\"]")
    if params.get("white_stem", False) is None:
        raise StyxValidationError("`white_stem` must not be None")
    if not isinstance(params["white_stem"], bool):
        raise StyxValidationError(f'`white_stem` has the wrong type: Received `{type(params.get("white_stem", False))}` expected `bool`')


def v_5ttgen_hsvs_cargs(
    params: V5ttgenHsvsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("hsvs")
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("output", None))
    if params.get("template", None) is not None:
        cargs.extend([
            "-template",
            execution.input_file(params.get("template", None))
        ])
    if params.get("hippocampi", None) is not None:
        cargs.extend([
            "-hippocampi",
            params.get("hippocampi", None)
        ])
    if params.get("thalami", None) is not None:
        cargs.extend([
            "-thalami",
            params.get("thalami", None)
        ])
    if params.get("white_stem", False):
        cargs.append("-white_stem")
    return cargs


def v_5ttgen_hsvs_outputs(
    params: V5ttgenHsvsParamsDict,
    execution: Execution,
) -> V5ttgenHsvsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = V5ttgenHsvsOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def v_5ttgen_config(
    key: str,
    value: str,
) -> V5ttgenConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def v_5ttgen_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `V5ttgenConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def v_5ttgen_config_cargs(
    params: V5ttgenConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class V5ttgenOutputs(typing.NamedTuple):
    """
    Output object returned when calling `V5ttgenParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    algorithm: typing.Union[V5ttgenFreesurferOutputs, V5ttgenFslOutputs, V5ttgenGifOutputs, V5ttgenHsvsOutputs]
    """Outputs from `V5ttgenFreesurferParamsDict` or `V5ttgenFslParamsDict` or
    `V5ttgenGifParamsDict` or `V5ttgenHsvsParamsDict`."""


def v_5ttgen_params(
    algorithm: typing.Union[V5ttgenFreesurferParamsDictTagged, V5ttgenFslParamsDictTagged, V5ttgenGifParamsDictTagged, V5ttgenHsvsParamsDictTagged],
    nocrop: bool = False,
    sgm_amyg_hipp: bool = False,
    nocleanup: bool = False,
    scratch: str | None = None,
    continue_: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[V5ttgenConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> V5ttgenParamsDictTagged:
    """
    Build parameters.
    
    Args:
        algorithm: Select the algorithm to be used to complete the script\
            operation; additional details and options become available once an\
            algorithm is nominated. Options are: freesurfer, fsl, gif, hsvs.
        nocrop: Do NOT crop the resulting 5TT image to reduce its size (keep\
            the same dimensions as the input image).
        sgm_amyg_hipp: Represent the amygdalae and hippocampi as sub-cortical\
            grey matter in the 5TT image.
        nocleanup: do not delete intermediate files during script execution,\
            and do not delete scratch directory at script completion.
        scratch: manually specify the path in which to generate the scratch\
            directory.
        continue_: continue the script from a previous execution; must provide\
            the scratch directory path, and the name of the last\
            successfully-generated file.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/5ttgen",
        "algorithm": algorithm,
        "nocrop": nocrop,
        "sgm_amyg_hipp": sgm_amyg_hipp,
        "nocleanup": nocleanup,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
    }
    if scratch is not None:
        params["scratch"] = scratch
    if continue_ is not None:
        params["continue"] = continue_
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def v_5ttgen_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `V5ttgenParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("algorithm", None) is None:
        raise StyxValidationError("`algorithm` must not be None")
    if not isinstance(params["algorithm"], dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params["algorithm"])}\'')
    if "@type" not in params["algorithm"]:
        raise StyxValidationError("Params object is missing `@type`")
    if params["algorithm"]["@type"] not in ["freesurfer", "fsl", "gif", "hsvs"]:
        raise StyxValidationError("Parameter `algorithm`s `@type` must be one of [\"freesurfer\", \"fsl\", \"gif\", \"hsvs\"]")
    v_5ttgen_algorithm_validate_dyn_fn(params["algorithm"]["@type"])(params["algorithm"])
    if params.get("nocrop", False) is None:
        raise StyxValidationError("`nocrop` must not be None")
    if not isinstance(params["nocrop"], bool):
        raise StyxValidationError(f'`nocrop` has the wrong type: Received `{type(params.get("nocrop", False))}` expected `bool`')
    if params.get("sgm_amyg_hipp", False) is None:
        raise StyxValidationError("`sgm_amyg_hipp` must not be None")
    if not isinstance(params["sgm_amyg_hipp"], bool):
        raise StyxValidationError(f'`sgm_amyg_hipp` has the wrong type: Received `{type(params.get("sgm_amyg_hipp", False))}` expected `bool`')
    if params.get("nocleanup", False) is None:
        raise StyxValidationError("`nocleanup` must not be None")
    if not isinstance(params["nocleanup"], bool):
        raise StyxValidationError(f'`nocleanup` has the wrong type: Received `{type(params.get("nocleanup", False))}` expected `bool`')
    if params.get("scratch", None) is not None:
        if not isinstance(params["scratch"], str):
            raise StyxValidationError(f'`scratch` has the wrong type: Received `{type(params.get("scratch", None))}` expected `str | None`')
    if params.get("continue", None) is not None:
        if not isinstance(params["continue"], str):
            raise StyxValidationError(f'`continue` has the wrong type: Received `{type(params.get("continue", None))}` expected `str | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[V5ttgenConfigParamsDict] | None`')
        for e in params["config"]:
            v_5ttgen_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')


def v_5ttgen_cargs(
    params: V5ttgenParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("5ttgen")
    cargs.extend(v_5ttgen_algorithm_cargs_dyn_fn(params.get("algorithm", None)["@type"])(params.get("algorithm", None), execution))
    if params.get("nocrop", False):
        cargs.append("-nocrop")
    if params.get("sgm_amyg_hipp", False):
        cargs.append("-sgm_amyg_hipp")
    if params.get("nocleanup", False):
        cargs.append("-nocleanup")
    if params.get("scratch", None) is not None:
        cargs.extend([
            "-scratch",
            params.get("scratch", None)
        ])
    if params.get("continue", None) is not None:
        cargs.extend([
            "-continue",
            params.get("continue", None)
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [v_5ttgen_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    return cargs


def v_5ttgen_outputs(
    params: V5ttgenParamsDict,
    execution: Execution,
) -> V5ttgenOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = V5ttgenOutputs(
        root=execution.output_file("."),
        algorithm=v_5ttgen_algorithm_outputs_dyn_fn(params.get("algorithm")["@type"])(params.get("algorithm"), execution),
    )
    return ret


def v_5ttgen_execute(
    params: V5ttgenParamsDict,
    runner: Runner | None = None,
) -> V5ttgenOutputs:
    """
    5ttgen
    
    Generate a 5TT image suitable for ACT.
    
    5ttgen acts as a 'master' script for generating a five-tissue-type (5TT)
    segmented tissue image suitable for use in Anatomically-Constrained
    Tractography (ACT). A range of different algorithms are available for
    completing this task. When using this script, the name of the algorithm to
    be used must appear as the first argument on the command-line after
    '5ttgen'. The subsequent compulsory arguments and options available depend
    on the particular algorithm being invoked.
    Each algorithm available also has its own help page, including necessary
    references; e.g. to see the help page of the 'fsl' algorithm, type '5ttgen
    fsl'.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V5ttgenOutputs`).
    """
    v_5ttgen_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_5TTGEN_METADATA)
    params = execution.params(params)
    cargs = v_5ttgen_cargs(params, execution)
    ret = v_5ttgen_outputs(params, execution)
    execution.run(cargs)
    return ret


def v_5ttgen(
    algorithm: typing.Union[V5ttgenFreesurferParamsDictTagged, V5ttgenFslParamsDictTagged, V5ttgenGifParamsDictTagged, V5ttgenHsvsParamsDictTagged],
    nocrop: bool = False,
    sgm_amyg_hipp: bool = False,
    nocleanup: bool = False,
    scratch: str | None = None,
    continue_: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[V5ttgenConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> V5ttgenOutputs:
    """
    5ttgen
    
    Generate a 5TT image suitable for ACT.
    
    5ttgen acts as a 'master' script for generating a five-tissue-type (5TT)
    segmented tissue image suitable for use in Anatomically-Constrained
    Tractography (ACT). A range of different algorithms are available for
    completing this task. When using this script, the name of the algorithm to
    be used must appear as the first argument on the command-line after
    '5ttgen'. The subsequent compulsory arguments and options available depend
    on the particular algorithm being invoked.
    Each algorithm available also has its own help page, including necessary
    references; e.g. to see the help page of the 'fsl' algorithm, type '5ttgen
    fsl'.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        algorithm: Select the algorithm to be used to complete the script\
            operation; additional details and options become available once an\
            algorithm is nominated. Options are: freesurfer, fsl, gif, hsvs.
        nocrop: Do NOT crop the resulting 5TT image to reduce its size (keep\
            the same dimensions as the input image).
        sgm_amyg_hipp: Represent the amygdalae and hippocampi as sub-cortical\
            grey matter in the 5TT image.
        nocleanup: do not delete intermediate files during script execution,\
            and do not delete scratch directory at script completion.
        scratch: manually specify the path in which to generate the scratch\
            directory.
        continue_: continue the script from a previous execution; must provide\
            the scratch directory path, and the name of the last\
            successfully-generated file.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V5ttgenOutputs`).
    """
    params = v_5ttgen_params(
        algorithm=algorithm,
        nocrop=nocrop,
        sgm_amyg_hipp=sgm_amyg_hipp,
        nocleanup=nocleanup,
        scratch=scratch,
        continue_=continue_,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
    )
    return v_5ttgen_execute(params, runner)


__all__ = [
    "V5ttgenConfigParamsDict",
    "V5ttgenConfigParamsDictTagged",
    "V5ttgenFreesurferOutputs",
    "V5ttgenFreesurferParamsDict",
    "V5ttgenFreesurferParamsDictTagged",
    "V5ttgenFslOutputs",
    "V5ttgenFslParamsDict",
    "V5ttgenFslParamsDictTagged",
    "V5ttgenGifOutputs",
    "V5ttgenGifParamsDict",
    "V5ttgenGifParamsDictTagged",
    "V5ttgenHsvsOutputs",
    "V5ttgenHsvsParamsDict",
    "V5ttgenHsvsParamsDictTagged",
    "V5ttgenOutputs",
    "V5ttgenParamsDict",
    "V5ttgenParamsDictTagged",
    "V_5TTGEN_METADATA",
    "v_5ttgen",
    "v_5ttgen_config",
    "v_5ttgen_execute",
    "v_5ttgen_freesurfer",
    "v_5ttgen_fsl",
    "v_5ttgen_gif",
    "v_5ttgen_hsvs",
    "v_5ttgen_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

WARP2METRIC_METADATA = Metadata(
    id="6239916d39fe990e43c4b408dab7798207712e0c.boutiques",
    name="warp2metric",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_Warp2metricFcParamsDictNoTag = typing.TypedDict('_Warp2metricFcParamsDictNoTag', {
    "template_fixel_directory": InputPathType,
    "output_fixel_directory": str,
    "output_fixel_data": str,
})
Warp2metricFcParamsDictTagged = typing.TypedDict('Warp2metricFcParamsDictTagged', {
    "@type": typing.Literal["fc"],
    "template_fixel_directory": InputPathType,
    "output_fixel_directory": str,
    "output_fixel_data": str,
})
Warp2metricFcParamsDict = _Warp2metricFcParamsDictNoTag | Warp2metricFcParamsDictTagged


_Warp2metricConfigParamsDictNoTag = typing.TypedDict('_Warp2metricConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
Warp2metricConfigParamsDictTagged = typing.TypedDict('Warp2metricConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
Warp2metricConfigParamsDict = _Warp2metricConfigParamsDictNoTag | Warp2metricConfigParamsDictTagged


_Warp2metricParamsDictNoTag = typing.TypedDict('_Warp2metricParamsDictNoTag', {
    "fc": typing.NotRequired[Warp2metricFcParamsDict | None],
    "jmat": typing.NotRequired[str | None],
    "jdet": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Warp2metricConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "in": InputPathType,
})
Warp2metricParamsDictTagged = typing.TypedDict('Warp2metricParamsDictTagged', {
    "@type": typing.Literal["mrtrix/warp2metric"],
    "fc": typing.NotRequired[Warp2metricFcParamsDict | None],
    "jmat": typing.NotRequired[str | None],
    "jdet": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Warp2metricConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "in": InputPathType,
})
Warp2metricParamsDict = _Warp2metricParamsDictNoTag | Warp2metricParamsDictTagged


def warp2metric_fc(
    template_fixel_directory: InputPathType,
    output_fixel_directory: str,
    output_fixel_data: str,
) -> Warp2metricFcParamsDictTagged:
    """
    Build parameters.
    
    Args:
        template_fixel_directory: use an input template fixel image to define\
            fibre orientations and output a fixel image describing the change in\
            fibre cross-section (FC) in the perpendicular plane to the fixel\
            orientation. e.g. warp2metric warp.mif -fc fixel_template_directory\
            output_fixel_directory fc.mif.
        output_fixel_directory: use an input template fixel image to define\
            fibre orientations and output a fixel image describing the change in\
            fibre cross-section (FC) in the perpendicular plane to the fixel\
            orientation. e.g. warp2metric warp.mif -fc fixel_template_directory\
            output_fixel_directory fc.mif.
        output_fixel_data: use an input template fixel image to define fibre\
            orientations and output a fixel image describing the change in fibre\
            cross-section (FC) in the perpendicular plane to the fixel orientation.\
            e.g. warp2metric warp.mif -fc fixel_template_directory\
            output_fixel_directory fc.mif.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "fc",
        "template_fixel_directory": template_fixel_directory,
        "output_fixel_directory": output_fixel_directory,
        "output_fixel_data": output_fixel_data,
    }
    return params


def warp2metric_fc_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Warp2metricFcParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("template_fixel_directory", None) is None:
        raise StyxValidationError("`template_fixel_directory` must not be None")
    if not isinstance(params["template_fixel_directory"], (pathlib.Path, str)):
        raise StyxValidationError(f'`template_fixel_directory` has the wrong type: Received `{type(params.get("template_fixel_directory", None))}` expected `InputPathType`')
    if params.get("output_fixel_directory", None) is None:
        raise StyxValidationError("`output_fixel_directory` must not be None")
    if not isinstance(params["output_fixel_directory"], str):
        raise StyxValidationError(f'`output_fixel_directory` has the wrong type: Received `{type(params.get("output_fixel_directory", None))}` expected `str`')
    if params.get("output_fixel_data", None) is None:
        raise StyxValidationError("`output_fixel_data` must not be None")
    if not isinstance(params["output_fixel_data"], str):
        raise StyxValidationError(f'`output_fixel_data` has the wrong type: Received `{type(params.get("output_fixel_data", None))}` expected `str`')


def warp2metric_fc_cargs(
    params: Warp2metricFcParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-fc")
    cargs.append(execution.input_file(params.get("template_fixel_directory", None)))
    cargs.append(params.get("output_fixel_directory", None))
    cargs.append(params.get("output_fixel_data", None))
    return cargs


def warp2metric_config(
    key: str,
    value: str,
) -> Warp2metricConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def warp2metric_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Warp2metricConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def warp2metric_config_cargs(
    params: Warp2metricConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class Warp2metricOutputs(typing.NamedTuple):
    """
    Output object returned when calling `Warp2metricParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    jmat: OutputPathType | None
    """output a Jacobian matrix image stored in column-major order along the 4th
    dimension.Note the output jacobian describes the warp gradient w.r.t the
    scanner space coordinate system """
    jdet: OutputPathType | None
    """output the Jacobian determinant instead of the full matrix """


def warp2metric_params(
    in_: InputPathType,
    fc: Warp2metricFcParamsDict | None = None,
    jmat: str | None = None,
    jdet: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Warp2metricConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Warp2metricParamsDictTagged:
    """
    Build parameters.
    
    Args:
        in_: the input deformation field.
        fc: use an input template fixel image to define fibre orientations and\
            output a fixel image describing the change in fibre cross-section (FC)\
            in the perpendicular plane to the fixel orientation. e.g. warp2metric\
            warp.mif -fc fixel_template_directory output_fixel_directory fc.mif.
        jmat: output a Jacobian matrix image stored in column-major order along\
            the 4th dimension.Note the output jacobian describes the warp gradient\
            w.r.t the scanner space coordinate system.
        jdet: output the Jacobian determinant instead of the full matrix.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/warp2metric",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "in": in_,
    }
    if fc is not None:
        params["fc"] = fc
    if jmat is not None:
        params["jmat"] = jmat
    if jdet is not None:
        params["jdet"] = jdet
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def warp2metric_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Warp2metricParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("fc", None) is not None:
        warp2metric_fc_validate(params["fc"])
    if params.get("jmat", None) is not None:
        if not isinstance(params["jmat"], str):
            raise StyxValidationError(f'`jmat` has the wrong type: Received `{type(params.get("jmat", None))}` expected `str | None`')
    if params.get("jdet", None) is not None:
        if not isinstance(params["jdet"], str):
            raise StyxValidationError(f'`jdet` has the wrong type: Received `{type(params.get("jdet", None))}` expected `str | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[Warp2metricConfigParamsDict] | None`')
        for e in params["config"]:
            warp2metric_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("in", None) is None:
        raise StyxValidationError("`in` must not be None")
    if not isinstance(params["in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`in` has the wrong type: Received `{type(params.get("in", None))}` expected `InputPathType`')


def warp2metric_cargs(
    params: Warp2metricParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("warp2metric")
    if params.get("fc", None) is not None:
        cargs.extend(warp2metric_fc_cargs(params.get("fc", None), execution))
    if params.get("jmat", None) is not None:
        cargs.extend([
            "-jmat",
            params.get("jmat", None)
        ])
    if params.get("jdet", None) is not None:
        cargs.extend([
            "-jdet",
            params.get("jdet", None)
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [warp2metric_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("in", None)))
    return cargs


def warp2metric_outputs(
    params: Warp2metricParamsDict,
    execution: Execution,
) -> Warp2metricOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Warp2metricOutputs(
        root=execution.output_file("."),
        jmat=execution.output_file(params.get("jmat", None)) if (params.get("jmat") is not None) else None,
        jdet=execution.output_file(params.get("jdet", None)) if (params.get("jdet") is not None) else None,
    )
    return ret


def warp2metric_execute(
    params: Warp2metricParamsDict,
    runner: Runner | None = None,
) -> Warp2metricOutputs:
    """
    warp2metric
    
    Compute fixel-wise or voxel-wise metrics from a 4D deformation field.
    
    
    
    References:
    
    Raffelt, D.; Tournier, JD/; Smith, RE.; Vaughan, DN.; Jackson, G.; Ridgway,
    GR. Connelly, A.Investigating White Matter Fibre Density and Morphology
    using Fixel-Based Analysis. Neuroimage, 2017, 144, 58-73, doi:
    10.1016/j.neuroimage.2016.09.029.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Warp2metricOutputs`).
    """
    warp2metric_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(WARP2METRIC_METADATA)
    params = execution.params(params)
    cargs = warp2metric_cargs(params, execution)
    ret = warp2metric_outputs(params, execution)
    execution.run(cargs)
    return ret


def warp2metric(
    in_: InputPathType,
    fc: Warp2metricFcParamsDict | None = None,
    jmat: str | None = None,
    jdet: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Warp2metricConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Warp2metricOutputs:
    """
    warp2metric
    
    Compute fixel-wise or voxel-wise metrics from a 4D deformation field.
    
    
    
    References:
    
    Raffelt, D.; Tournier, JD/; Smith, RE.; Vaughan, DN.; Jackson, G.; Ridgway,
    GR. Connelly, A.Investigating White Matter Fibre Density and Morphology
    using Fixel-Based Analysis. Neuroimage, 2017, 144, 58-73, doi:
    10.1016/j.neuroimage.2016.09.029.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        in_: the input deformation field.
        fc: use an input template fixel image to define fibre orientations and\
            output a fixel image describing the change in fibre cross-section (FC)\
            in the perpendicular plane to the fixel orientation. e.g. warp2metric\
            warp.mif -fc fixel_template_directory output_fixel_directory fc.mif.
        jmat: output a Jacobian matrix image stored in column-major order along\
            the 4th dimension.Note the output jacobian describes the warp gradient\
            w.r.t the scanner space coordinate system.
        jdet: output the Jacobian determinant instead of the full matrix.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Warp2metricOutputs`).
    """
    params = warp2metric_params(
        fc=fc,
        jmat=jmat,
        jdet=jdet,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        in_=in_,
    )
    return warp2metric_execute(params, runner)


__all__ = [
    "WARP2METRIC_METADATA",
    "Warp2metricConfigParamsDict",
    "Warp2metricConfigParamsDictTagged",
    "Warp2metricFcParamsDict",
    "Warp2metricFcParamsDictTagged",
    "Warp2metricOutputs",
    "Warp2metricParamsDict",
    "Warp2metricParamsDictTagged",
    "warp2metric",
    "warp2metric_config",
    "warp2metric_execute",
    "warp2metric_fc",
    "warp2metric_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SHBASIS_METADATA = Metadata(
    id="8d1c3a9e142d8ac3b0cfbca9e0fcd6b3d41e317e.boutiques",
    name="shbasis",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_ShbasisConfigParamsDictNoTag = typing.TypedDict('_ShbasisConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
ShbasisConfigParamsDictTagged = typing.TypedDict('ShbasisConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
ShbasisConfigParamsDict = _ShbasisConfigParamsDictNoTag | ShbasisConfigParamsDictTagged


_ShbasisParamsDictNoTag = typing.TypedDict('_ShbasisParamsDictNoTag', {
    "convert": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[ShbasisConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "SH": list[InputPathType],
})
ShbasisParamsDictTagged = typing.TypedDict('ShbasisParamsDictTagged', {
    "@type": typing.Literal["mrtrix/shbasis"],
    "convert": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[ShbasisConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "SH": list[InputPathType],
})
ShbasisParamsDict = _ShbasisParamsDictNoTag | ShbasisParamsDictTagged


def shbasis_config(
    key: str,
    value: str,
) -> ShbasisConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def shbasis_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ShbasisConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def shbasis_config_cargs(
    params: ShbasisConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class ShbasisOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ShbasisParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def shbasis_params(
    sh: list[InputPathType],
    convert: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[ShbasisConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> ShbasisParamsDictTagged:
    """
    Build parameters.
    
    Args:
        sh: the input image(s) of SH coefficients.
        convert: convert the image data in-place to the desired basis; options\
            are: old,new,force_oldtonew,force_newtoold.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/shbasis",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "SH": sh,
    }
    if convert is not None:
        params["convert"] = convert
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def shbasis_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ShbasisParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("convert", None) is not None:
        if not isinstance(params["convert"], str):
            raise StyxValidationError(f'`convert` has the wrong type: Received `{type(params.get("convert", None))}` expected `str | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[ShbasisConfigParamsDict] | None`')
        for e in params["config"]:
            shbasis_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("SH", None) is None:
        raise StyxValidationError("`SH` must not be None")
    if not isinstance(params["SH"], list):
        raise StyxValidationError(f'`SH` has the wrong type: Received `{type(params.get("SH", None))}` expected `list[InputPathType]`')
    for e in params["SH"]:
        if not isinstance(e, (pathlib.Path, str)):
            raise StyxValidationError(f'`SH` has the wrong type: Received `{type(params.get("SH", None))}` expected `list[InputPathType]`')


def shbasis_cargs(
    params: ShbasisParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("shbasis")
    if params.get("convert", None) is not None:
        cargs.extend([
            "-convert",
            params.get("convert", None)
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [shbasis_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.extend([execution.input_file(f) for f in params.get("SH", None)])
    return cargs


def shbasis_outputs(
    params: ShbasisParamsDict,
    execution: Execution,
) -> ShbasisOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ShbasisOutputs(
        root=execution.output_file("."),
    )
    return ret


def shbasis_execute(
    params: ShbasisParamsDict,
    runner: Runner | None = None,
) -> ShbasisOutputs:
    """
    shbasis
    
    Examine the values in spherical harmonic images to estimate (and optionally
    change) the SH basis used.
    
    In previous versions of MRtrix, the convention used for storing spherical
    harmonic coefficients was a non-orthonormal basis (the m!=0 coefficients
    were a factor of sqrt(2) too large). This error has been rectified in newer
    versions of MRtrix, but will cause issues if processing SH data that was
    generated using an older version of MRtrix (or vice-versa).
    
    This command provides a mechanism for testing the basis used in storage of
    image data representing a spherical harmonic series per voxel, and allows
    the user to forcibly modify the raw image data to conform to the desired
    basis.
    
    Note that the "force_*" conversion choices should only be used in cases
    where this command has previously been unable to automatically determine the
    SH basis from the image data, but the user themselves are confident of the
    SH basis of the data.
    
    The spherical harmonic coefficients are stored according the conventions
    described the main documentation, which can be found at the following link:
    https://mrtrix.readthedocs.io/en/3.0.4/concepts/spherical_harmonics.html
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ShbasisOutputs`).
    """
    shbasis_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SHBASIS_METADATA)
    params = execution.params(params)
    cargs = shbasis_cargs(params, execution)
    ret = shbasis_outputs(params, execution)
    execution.run(cargs)
    return ret


def shbasis(
    sh: list[InputPathType],
    convert: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[ShbasisConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> ShbasisOutputs:
    """
    shbasis
    
    Examine the values in spherical harmonic images to estimate (and optionally
    change) the SH basis used.
    
    In previous versions of MRtrix, the convention used for storing spherical
    harmonic coefficients was a non-orthonormal basis (the m!=0 coefficients
    were a factor of sqrt(2) too large). This error has been rectified in newer
    versions of MRtrix, but will cause issues if processing SH data that was
    generated using an older version of MRtrix (or vice-versa).
    
    This command provides a mechanism for testing the basis used in storage of
    image data representing a spherical harmonic series per voxel, and allows
    the user to forcibly modify the raw image data to conform to the desired
    basis.
    
    Note that the "force_*" conversion choices should only be used in cases
    where this command has previously been unable to automatically determine the
    SH basis from the image data, but the user themselves are confident of the
    SH basis of the data.
    
    The spherical harmonic coefficients are stored according the conventions
    described the main documentation, which can be found at the following link:
    https://mrtrix.readthedocs.io/en/3.0.4/concepts/spherical_harmonics.html
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        sh: the input image(s) of SH coefficients.
        convert: convert the image data in-place to the desired basis; options\
            are: old,new,force_oldtonew,force_newtoold.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ShbasisOutputs`).
    """
    params = shbasis_params(
        convert=convert,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        sh=sh,
    )
    return shbasis_execute(params, runner)


__all__ = [
    "SHBASIS_METADATA",
    "ShbasisConfigParamsDict",
    "ShbasisConfigParamsDictTagged",
    "ShbasisOutputs",
    "ShbasisParamsDict",
    "ShbasisParamsDictTagged",
    "shbasis",
    "shbasis_config",
    "shbasis_execute",
    "shbasis_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

FIXEL2VOXEL_METADATA = Metadata(
    id="0e517d0eb23dea1741bca5bac37fc1918bda4919.boutiques",
    name="fixel2voxel",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_Fixel2voxelConfigParamsDictNoTag = typing.TypedDict('_Fixel2voxelConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
Fixel2voxelConfigParamsDictTagged = typing.TypedDict('Fixel2voxelConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
Fixel2voxelConfigParamsDict = _Fixel2voxelConfigParamsDictNoTag | Fixel2voxelConfigParamsDictTagged


_Fixel2voxelParamsDictNoTag = typing.TypedDict('_Fixel2voxelParamsDictNoTag', {
    "number": typing.NotRequired[int | None],
    "fill": typing.NotRequired[float | None],
    "weighted": typing.NotRequired[InputPathType | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Fixel2voxelConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "fixel_in": InputPathType,
    "operation": str,
    "image_out": str,
})
Fixel2voxelParamsDictTagged = typing.TypedDict('Fixel2voxelParamsDictTagged', {
    "@type": typing.Literal["mrtrix/fixel2voxel"],
    "number": typing.NotRequired[int | None],
    "fill": typing.NotRequired[float | None],
    "weighted": typing.NotRequired[InputPathType | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Fixel2voxelConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "fixel_in": InputPathType,
    "operation": str,
    "image_out": str,
})
Fixel2voxelParamsDict = _Fixel2voxelParamsDictNoTag | Fixel2voxelParamsDictTagged


def fixel2voxel_config(
    key: str,
    value: str,
) -> Fixel2voxelConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def fixel2voxel_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Fixel2voxelConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def fixel2voxel_config_cargs(
    params: Fixel2voxelConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class Fixel2voxelOutputs(typing.NamedTuple):
    """
    Output object returned when calling `Fixel2voxelParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    image_out: OutputPathType
    """the output scalar image."""


def fixel2voxel_params(
    fixel_in: InputPathType,
    operation: str,
    image_out: str,
    number: int | None = None,
    fill: float | None = None,
    weighted: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Fixel2voxelConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Fixel2voxelParamsDictTagged:
    """
    Build parameters.
    
    Args:
        fixel_in: the input fixel data file.
        operation: the operation to apply, one of: mean, sum, product, min,\
            max, absmax, magmax, count, complexity, sf, dec_unit, dec_scaled, none.
        image_out: the output scalar image.
        number: use only the largest N fixels in calculation of the voxel-wise\
            statistic; in the case of operation "none", output only the largest N\
            fixels in each voxel.
        fill: for "none" operation, specify the value to fill when number of\
            fixels is fewer than the maximum (default: 0.0).
        weighted: weight the contribution of each fixel to the per-voxel result\
            according to its volume.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/fixel2voxel",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "fixel_in": fixel_in,
        "operation": operation,
        "image_out": image_out,
    }
    if number is not None:
        params["number"] = number
    if fill is not None:
        params["fill"] = fill
    if weighted is not None:
        params["weighted"] = weighted
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def fixel2voxel_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Fixel2voxelParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("number", None) is not None:
        if not isinstance(params["number"], int):
            raise StyxValidationError(f'`number` has the wrong type: Received `{type(params.get("number", None))}` expected `int | None`')
    if params.get("fill", None) is not None:
        if not isinstance(params["fill"], (float, int)):
            raise StyxValidationError(f'`fill` has the wrong type: Received `{type(params.get("fill", None))}` expected `float | None`')
    if params.get("weighted", None) is not None:
        if not isinstance(params["weighted"], (pathlib.Path, str)):
            raise StyxValidationError(f'`weighted` has the wrong type: Received `{type(params.get("weighted", None))}` expected `InputPathType | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[Fixel2voxelConfigParamsDict] | None`')
        for e in params["config"]:
            fixel2voxel_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("fixel_in", None) is None:
        raise StyxValidationError("`fixel_in` must not be None")
    if not isinstance(params["fixel_in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`fixel_in` has the wrong type: Received `{type(params.get("fixel_in", None))}` expected `InputPathType`')
    if params.get("operation", None) is None:
        raise StyxValidationError("`operation` must not be None")
    if not isinstance(params["operation"], str):
        raise StyxValidationError(f'`operation` has the wrong type: Received `{type(params.get("operation", None))}` expected `str`')
    if params.get("image_out", None) is None:
        raise StyxValidationError("`image_out` must not be None")
    if not isinstance(params["image_out"], str):
        raise StyxValidationError(f'`image_out` has the wrong type: Received `{type(params.get("image_out", None))}` expected `str`')


def fixel2voxel_cargs(
    params: Fixel2voxelParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("fixel2voxel")
    if params.get("number", None) is not None:
        cargs.extend([
            "-number",
            str(params.get("number", None))
        ])
    if params.get("fill", None) is not None:
        cargs.extend([
            "-fill",
            str(params.get("fill", None))
        ])
    if params.get("weighted", None) is not None:
        cargs.extend([
            "-weighted",
            execution.input_file(params.get("weighted", None))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [fixel2voxel_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("fixel_in", None)))
    cargs.append(params.get("operation", None))
    cargs.append(params.get("image_out", None))
    return cargs


def fixel2voxel_outputs(
    params: Fixel2voxelParamsDict,
    execution: Execution,
) -> Fixel2voxelOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Fixel2voxelOutputs(
        root=execution.output_file("."),
        image_out=execution.output_file(params.get("image_out", None)),
    )
    return ret


def fixel2voxel_execute(
    params: Fixel2voxelParamsDict,
    runner: Runner | None = None,
) -> Fixel2voxelOutputs:
    """
    fixel2voxel
    
    Convert a fixel-based sparse-data image into some form of scalar image.
    
    Fixel data can be reduced to voxel data in a number of ways:
    
    - Some statistic computed across all fixel values within a voxel: mean, sum,
    product, min, max, absmax, magmax
    
    - The number of fixels in each voxel: count
    
    - Some measure of crossing-fibre organisation: complexity, sf
    ('single-fibre')
    
    - A 4D directionally-encoded colour image: dec_unit, dec_scaled
    
    - A 4D image containing all fixel data values in each voxel unmodified: none
    
    The -weighted option deals with the case where there is some per-fixel
    metric of interest that you wish to collapse into a single scalar measure
    per voxel, but each fixel possesses a different volume, and you wish for
    those fixels with greater volume to have a greater influence on the
    calculation than fixels with lesser volume. For instance, when estimating a
    voxel-based measure of mean axon diameter from per-fixel mean axon
    diameters, a fixel's mean axon diameter should be weigthed by its relative
    volume within the voxel in the calculation of that voxel mean.
    
    References:
    
    * Reference for 'complexity' operation:
    Riffert, T. W.; Schreiber, J.; Anwander, A. & Knosche, T. R. Beyond
    Fractional Anisotropy: Extraction of bundle-specific structural metrics from
    crossing fibre models. NeuroImage, 2014, 100, 176-191.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Fixel2voxelOutputs`).
    """
    fixel2voxel_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(FIXEL2VOXEL_METADATA)
    params = execution.params(params)
    cargs = fixel2voxel_cargs(params, execution)
    ret = fixel2voxel_outputs(params, execution)
    execution.run(cargs)
    return ret


def fixel2voxel(
    fixel_in: InputPathType,
    operation: str,
    image_out: str,
    number: int | None = None,
    fill: float | None = None,
    weighted: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Fixel2voxelConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Fixel2voxelOutputs:
    """
    fixel2voxel
    
    Convert a fixel-based sparse-data image into some form of scalar image.
    
    Fixel data can be reduced to voxel data in a number of ways:
    
    - Some statistic computed across all fixel values within a voxel: mean, sum,
    product, min, max, absmax, magmax
    
    - The number of fixels in each voxel: count
    
    - Some measure of crossing-fibre organisation: complexity, sf
    ('single-fibre')
    
    - A 4D directionally-encoded colour image: dec_unit, dec_scaled
    
    - A 4D image containing all fixel data values in each voxel unmodified: none
    
    The -weighted option deals with the case where there is some per-fixel
    metric of interest that you wish to collapse into a single scalar measure
    per voxel, but each fixel possesses a different volume, and you wish for
    those fixels with greater volume to have a greater influence on the
    calculation than fixels with lesser volume. For instance, when estimating a
    voxel-based measure of mean axon diameter from per-fixel mean axon
    diameters, a fixel's mean axon diameter should be weigthed by its relative
    volume within the voxel in the calculation of that voxel mean.
    
    References:
    
    * Reference for 'complexity' operation:
    Riffert, T. W.; Schreiber, J.; Anwander, A. & Knosche, T. R. Beyond
    Fractional Anisotropy: Extraction of bundle-specific structural metrics from
    crossing fibre models. NeuroImage, 2014, 100, 176-191.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        fixel_in: the input fixel data file.
        operation: the operation to apply, one of: mean, sum, product, min,\
            max, absmax, magmax, count, complexity, sf, dec_unit, dec_scaled, none.
        image_out: the output scalar image.
        number: use only the largest N fixels in calculation of the voxel-wise\
            statistic; in the case of operation "none", output only the largest N\
            fixels in each voxel.
        fill: for "none" operation, specify the value to fill when number of\
            fixels is fewer than the maximum (default: 0.0).
        weighted: weight the contribution of each fixel to the per-voxel result\
            according to its volume.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Fixel2voxelOutputs`).
    """
    params = fixel2voxel_params(
        number=number,
        fill=fill,
        weighted=weighted,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        fixel_in=fixel_in,
        operation=operation,
        image_out=image_out,
    )
    return fixel2voxel_execute(params, runner)


__all__ = [
    "FIXEL2VOXEL_METADATA",
    "Fixel2voxelConfigParamsDict",
    "Fixel2voxelConfigParamsDictTagged",
    "Fixel2voxelOutputs",
    "Fixel2voxelParamsDict",
    "Fixel2voxelParamsDictTagged",
    "fixel2voxel",
    "fixel2voxel_config",
    "fixel2voxel_execute",
    "fixel2voxel_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

FOD2DEC_METADATA = Metadata(
    id="56fd41eb31bb1240fd6341ac5abf21815b0e9dac.boutiques",
    name="fod2dec",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_Fod2decConfigParamsDictNoTag = typing.TypedDict('_Fod2decConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
Fod2decConfigParamsDictTagged = typing.TypedDict('Fod2decConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
Fod2decConfigParamsDict = _Fod2decConfigParamsDictNoTag | Fod2decConfigParamsDictTagged


_Fod2decParamsDictNoTag = typing.TypedDict('_Fod2decParamsDictNoTag', {
    "mask": typing.NotRequired[InputPathType | None],
    "contrast": typing.NotRequired[InputPathType | None],
    "lum": bool,
    "lum_coefs": typing.NotRequired[list[float] | None],
    "lum_gamma": typing.NotRequired[float | None],
    "threshold": typing.NotRequired[float | None],
    "no_weight": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Fod2decConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "output": str,
})
Fod2decParamsDictTagged = typing.TypedDict('Fod2decParamsDictTagged', {
    "@type": typing.Literal["mrtrix/fod2dec"],
    "mask": typing.NotRequired[InputPathType | None],
    "contrast": typing.NotRequired[InputPathType | None],
    "lum": bool,
    "lum_coefs": typing.NotRequired[list[float] | None],
    "lum_gamma": typing.NotRequired[float | None],
    "threshold": typing.NotRequired[float | None],
    "no_weight": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Fod2decConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "output": str,
})
Fod2decParamsDict = _Fod2decParamsDictNoTag | Fod2decParamsDictTagged


def fod2dec_config(
    key: str,
    value: str,
) -> Fod2decConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def fod2dec_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Fod2decConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def fod2dec_config_cargs(
    params: Fod2decConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class Fod2decOutputs(typing.NamedTuple):
    """
    Output object returned when calling `Fod2decParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """The output DEC image (weighted RGB triplets)."""


def fod2dec_params(
    input_: InputPathType,
    output: str,
    mask: InputPathType | None = None,
    contrast: InputPathType | None = None,
    lum: bool = False,
    lum_coefs: list[float] | None = None,
    lum_gamma: float | None = None,
    threshold: float | None = None,
    no_weight: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Fod2decConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Fod2decParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: The input FOD image (spherical harmonic coefficients).
        output: The output DEC image (weighted RGB triplets).
        mask: Only perform DEC computation within the specified mask image.
        contrast: Weight the computed DEC map by the provided image contrast.\
            If the contrast has a different image grid, the DEC map is first\
            resliced and renormalised. To achieve panchromatic sharpening, provide\
            an image with a higher spatial resolution than the input FOD image;\
            e.g., a T1 anatomical volume. Only the DEC is subject to the mask, so\
            as to allow for partial colouring of the contrast image.\
            Default when this option is *not* provided: integral of input FOD,\
            subject to the same mask/threshold as used for DEC computation.
        lum: Correct for luminance/perception, using default values Cr,Cg,Cb =\
            0.3,0.5,0.2 and gamma = 2.2 (*not* correcting is the theoretical\
            equivalent of Cr,Cg,Cb = 1,1,1 and gamma = 2).
        lum_coefs: The coefficients Cr,Cg,Cb to correct for\
            luminance/perception.\
            Note: this implicitly switches on luminance/perception correction,\
            using a default gamma = 2.2 unless specified otherwise.
        lum_gamma: The gamma value to correct for luminance/perception.\
            Note: this implicitly switches on luminance/perception correction,\
            using a default Cr,Cg,Cb = 0.3,0.5,0.2 unless specified otherwise.
        threshold: FOD amplitudes below the threshold value are considered\
            zero.
        no_weight: Do not weight the DEC map; just output the unweighted\
            colours. Reslicing and renormalising of colours will still happen when\
            providing the -contrast option as a template.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/fod2dec",
        "lum": lum,
        "no_weight": no_weight,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
        "output": output,
    }
    if mask is not None:
        params["mask"] = mask
    if contrast is not None:
        params["contrast"] = contrast
    if lum_coefs is not None:
        params["lum_coefs"] = lum_coefs
    if lum_gamma is not None:
        params["lum_gamma"] = lum_gamma
    if threshold is not None:
        params["threshold"] = threshold
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def fod2dec_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Fod2decParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("mask", None) is not None:
        if not isinstance(params["mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType | None`')
    if params.get("contrast", None) is not None:
        if not isinstance(params["contrast"], (pathlib.Path, str)):
            raise StyxValidationError(f'`contrast` has the wrong type: Received `{type(params.get("contrast", None))}` expected `InputPathType | None`')
    if params.get("lum", False) is None:
        raise StyxValidationError("`lum` must not be None")
    if not isinstance(params["lum"], bool):
        raise StyxValidationError(f'`lum` has the wrong type: Received `{type(params.get("lum", False))}` expected `bool`')
    if params.get("lum_coefs", None) is not None:
        if not isinstance(params["lum_coefs"], list):
            raise StyxValidationError(f'`lum_coefs` has the wrong type: Received `{type(params.get("lum_coefs", None))}` expected `list[float] | None`')
        for e in params["lum_coefs"]:
            if not isinstance(e, (float, int)):
                raise StyxValidationError(f'`lum_coefs` has the wrong type: Received `{type(params.get("lum_coefs", None))}` expected `list[float] | None`')
    if params.get("lum_gamma", None) is not None:
        if not isinstance(params["lum_gamma"], (float, int)):
            raise StyxValidationError(f'`lum_gamma` has the wrong type: Received `{type(params.get("lum_gamma", None))}` expected `float | None`')
    if params.get("threshold", None) is not None:
        if not isinstance(params["threshold"], (float, int)):
            raise StyxValidationError(f'`threshold` has the wrong type: Received `{type(params.get("threshold", None))}` expected `float | None`')
    if params.get("no_weight", False) is None:
        raise StyxValidationError("`no_weight` must not be None")
    if not isinstance(params["no_weight"], bool):
        raise StyxValidationError(f'`no_weight` has the wrong type: Received `{type(params.get("no_weight", False))}` expected `bool`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[Fod2decConfigParamsDict] | None`')
        for e in params["config"]:
            fod2dec_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def fod2dec_cargs(
    params: Fod2decParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("fod2dec")
    if params.get("mask", None) is not None:
        cargs.extend([
            "-mask",
            execution.input_file(params.get("mask", None))
        ])
    if params.get("contrast", None) is not None:
        cargs.extend([
            "-contrast",
            execution.input_file(params.get("contrast", None))
        ])
    if params.get("lum", False):
        cargs.append("-lum")
    if params.get("lum_coefs", None) is not None:
        cargs.extend([
            "-lum_coefs",
            *map(str, params.get("lum_coefs", None))
        ])
    if params.get("lum_gamma", None) is not None:
        cargs.extend([
            "-lum_gamma",
            str(params.get("lum_gamma", None))
        ])
    if params.get("threshold", None) is not None:
        cargs.extend([
            "-threshold",
            str(params.get("threshold", None))
        ])
    if params.get("no_weight", False):
        cargs.append("-no_weight")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [fod2dec_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("output", None))
    return cargs


def fod2dec_outputs(
    params: Fod2decParamsDict,
    execution: Execution,
) -> Fod2decOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Fod2decOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def fod2dec_execute(
    params: Fod2decParamsDict,
    runner: Runner | None = None,
) -> Fod2decOutputs:
    """
    fod2dec
    
    Generate FOD-based DEC maps, with optional panchromatic sharpening and/or
    luminance/perception correction.
    
    By default, the FOD-based DEC is weighted by the integral of the FOD. To
    weight by another scalar map, use the -contrast option. This option can also
    be used for panchromatic sharpening, e.g., by supplying a T1 (or other
    sensible) anatomical volume with a higher spatial resolution.
    
    References:
    
    Dhollander T, Smith RE, Tournier JD, Jeurissen B, Connelly A. Time to move
    on: an FOD-based DEC map to replace DTI's trademark DEC FA. Proc Intl Soc
    Mag Reson Med, 2015, 23, 1027
    
    Dhollander T, Raffelt D, Smith RE, Connelly A. Panchromatic sharpening of
    FOD-based DEC maps by structural T1 information. Proc Intl Soc Mag Reson
    Med, 2015, 23, 566.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Fod2decOutputs`).
    """
    fod2dec_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(FOD2DEC_METADATA)
    params = execution.params(params)
    cargs = fod2dec_cargs(params, execution)
    ret = fod2dec_outputs(params, execution)
    execution.run(cargs)
    return ret


def fod2dec(
    input_: InputPathType,
    output: str,
    mask: InputPathType | None = None,
    contrast: InputPathType | None = None,
    lum: bool = False,
    lum_coefs: list[float] | None = None,
    lum_gamma: float | None = None,
    threshold: float | None = None,
    no_weight: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Fod2decConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Fod2decOutputs:
    """
    fod2dec
    
    Generate FOD-based DEC maps, with optional panchromatic sharpening and/or
    luminance/perception correction.
    
    By default, the FOD-based DEC is weighted by the integral of the FOD. To
    weight by another scalar map, use the -contrast option. This option can also
    be used for panchromatic sharpening, e.g., by supplying a T1 (or other
    sensible) anatomical volume with a higher spatial resolution.
    
    References:
    
    Dhollander T, Smith RE, Tournier JD, Jeurissen B, Connelly A. Time to move
    on: an FOD-based DEC map to replace DTI's trademark DEC FA. Proc Intl Soc
    Mag Reson Med, 2015, 23, 1027
    
    Dhollander T, Raffelt D, Smith RE, Connelly A. Panchromatic sharpening of
    FOD-based DEC maps by structural T1 information. Proc Intl Soc Mag Reson
    Med, 2015, 23, 566.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: The input FOD image (spherical harmonic coefficients).
        output: The output DEC image (weighted RGB triplets).
        mask: Only perform DEC computation within the specified mask image.
        contrast: Weight the computed DEC map by the provided image contrast.\
            If the contrast has a different image grid, the DEC map is first\
            resliced and renormalised. To achieve panchromatic sharpening, provide\
            an image with a higher spatial resolution than the input FOD image;\
            e.g., a T1 anatomical volume. Only the DEC is subject to the mask, so\
            as to allow for partial colouring of the contrast image.\
            Default when this option is *not* provided: integral of input FOD,\
            subject to the same mask/threshold as used for DEC computation.
        lum: Correct for luminance/perception, using default values Cr,Cg,Cb =\
            0.3,0.5,0.2 and gamma = 2.2 (*not* correcting is the theoretical\
            equivalent of Cr,Cg,Cb = 1,1,1 and gamma = 2).
        lum_coefs: The coefficients Cr,Cg,Cb to correct for\
            luminance/perception.\
            Note: this implicitly switches on luminance/perception correction,\
            using a default gamma = 2.2 unless specified otherwise.
        lum_gamma: The gamma value to correct for luminance/perception.\
            Note: this implicitly switches on luminance/perception correction,\
            using a default Cr,Cg,Cb = 0.3,0.5,0.2 unless specified otherwise.
        threshold: FOD amplitudes below the threshold value are considered\
            zero.
        no_weight: Do not weight the DEC map; just output the unweighted\
            colours. Reslicing and renormalising of colours will still happen when\
            providing the -contrast option as a template.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Fod2decOutputs`).
    """
    params = fod2dec_params(
        mask=mask,
        contrast=contrast,
        lum=lum,
        lum_coefs=lum_coefs,
        lum_gamma=lum_gamma,
        threshold=threshold,
        no_weight=no_weight,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        output=output,
    )
    return fod2dec_execute(params, runner)


__all__ = [
    "FOD2DEC_METADATA",
    "Fod2decConfigParamsDict",
    "Fod2decConfigParamsDictTagged",
    "Fod2decOutputs",
    "Fod2decParamsDict",
    "Fod2decParamsDictTagged",
    "fod2dec",
    "fod2dec_config",
    "fod2dec_execute",
    "fod2dec_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SH2AMP_METADATA = Metadata(
    id="845bbdcb825f577d4f4e931e8e0dd0e1f1f2de94.boutiques",
    name="sh2amp",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_Sh2ampFslgradParamsDictNoTag = typing.TypedDict('_Sh2ampFslgradParamsDictNoTag', {
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
Sh2ampFslgradParamsDictTagged = typing.TypedDict('Sh2ampFslgradParamsDictTagged', {
    "@type": typing.Literal["fslgrad"],
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
Sh2ampFslgradParamsDict = _Sh2ampFslgradParamsDictNoTag | Sh2ampFslgradParamsDictTagged


_Sh2ampVariousStringParamsDictNoTag = typing.TypedDict('_Sh2ampVariousStringParamsDictNoTag', {
    "obj": str,
})
Sh2ampVariousStringParamsDictTagged = typing.TypedDict('Sh2ampVariousStringParamsDictTagged', {
    "@type": typing.Literal["VariousString"],
    "obj": str,
})
Sh2ampVariousStringParamsDict = _Sh2ampVariousStringParamsDictNoTag | Sh2ampVariousStringParamsDictTagged


_Sh2ampVariousFileParamsDictNoTag = typing.TypedDict('_Sh2ampVariousFileParamsDictNoTag', {
    "obj": InputPathType,
})
Sh2ampVariousFileParamsDictTagged = typing.TypedDict('Sh2ampVariousFileParamsDictTagged', {
    "@type": typing.Literal["VariousFile"],
    "obj": InputPathType,
})
Sh2ampVariousFileParamsDict = _Sh2ampVariousFileParamsDictNoTag | Sh2ampVariousFileParamsDictTagged


_Sh2ampConfigParamsDictNoTag = typing.TypedDict('_Sh2ampConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
Sh2ampConfigParamsDictTagged = typing.TypedDict('Sh2ampConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
Sh2ampConfigParamsDict = _Sh2ampConfigParamsDictNoTag | Sh2ampConfigParamsDictTagged


_Sh2ampParamsDictNoTag = typing.TypedDict('_Sh2ampParamsDictNoTag', {
    "nonnegative": bool,
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[Sh2ampFslgradParamsDict | None],
    "strides": typing.NotRequired[typing.Union[Sh2ampVariousStringParamsDictTagged, Sh2ampVariousFileParamsDictTagged] | None],
    "datatype": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Sh2ampConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "directions": InputPathType,
    "output": str,
})
Sh2ampParamsDictTagged = typing.TypedDict('Sh2ampParamsDictTagged', {
    "@type": typing.Literal["mrtrix/sh2amp"],
    "nonnegative": bool,
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[Sh2ampFslgradParamsDict | None],
    "strides": typing.NotRequired[typing.Union[Sh2ampVariousStringParamsDictTagged, Sh2ampVariousFileParamsDictTagged] | None],
    "datatype": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Sh2ampConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "directions": InputPathType,
    "output": str,
})
Sh2ampParamsDict = _Sh2ampParamsDictNoTag | Sh2ampParamsDictTagged


def sh2amp_strides_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "VariousString": sh2amp_various_string_cargs,
        "VariousFile": sh2amp_various_file_cargs,
    }.get(t)


def sh2amp_strides_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "VariousString": sh2amp_various_string_validate,
        "VariousFile": sh2amp_various_file_validate,
    }.get(t)


def sh2amp_fslgrad(
    bvecs: InputPathType,
    bvals: InputPathType,
) -> Sh2ampFslgradParamsDictTagged:
    """
    Build parameters.
    
    Args:
        bvecs: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        bvals: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "fslgrad",
        "bvecs": bvecs,
        "bvals": bvals,
    }
    return params


def sh2amp_fslgrad_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Sh2ampFslgradParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("bvecs", None) is None:
        raise StyxValidationError("`bvecs` must not be None")
    if not isinstance(params["bvecs"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvecs` has the wrong type: Received `{type(params.get("bvecs", None))}` expected `InputPathType`')
    if params.get("bvals", None) is None:
        raise StyxValidationError("`bvals` must not be None")
    if not isinstance(params["bvals"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvals` has the wrong type: Received `{type(params.get("bvals", None))}` expected `InputPathType`')


def sh2amp_fslgrad_cargs(
    params: Sh2ampFslgradParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-fslgrad")
    cargs.append(execution.input_file(params.get("bvecs", None)))
    cargs.append(execution.input_file(params.get("bvals", None)))
    return cargs


def sh2amp_various_string(
    obj: str,
) -> Sh2ampVariousStringParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: String object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousString",
        "obj": obj,
    }
    return params


def sh2amp_various_string_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Sh2ampVariousStringParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], str):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `str`')


def sh2amp_various_string_cargs(
    params: Sh2ampVariousStringParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("obj", None))
    return cargs


def sh2amp_various_file(
    obj: InputPathType,
) -> Sh2ampVariousFileParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: File object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousFile",
        "obj": obj,
    }
    return params


def sh2amp_various_file_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Sh2ampVariousFileParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], (pathlib.Path, str)):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `InputPathType`')


def sh2amp_various_file_cargs(
    params: Sh2ampVariousFileParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("obj", None)))
    return cargs


def sh2amp_config(
    key: str,
    value: str,
) -> Sh2ampConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def sh2amp_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Sh2ampConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def sh2amp_config_cargs(
    params: Sh2ampConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class Sh2ampOutputs(typing.NamedTuple):
    """
    Output object returned when calling `Sh2ampParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output image consisting of the amplitude of the SH functions along
    the specified directions."""


def sh2amp_params(
    input_: InputPathType,
    directions: InputPathType,
    output: str,
    nonnegative: bool = False,
    grad: InputPathType | None = None,
    fslgrad: Sh2ampFslgradParamsDict | None = None,
    strides: typing.Union[Sh2ampVariousStringParamsDictTagged, Sh2ampVariousFileParamsDictTagged] | None = None,
    datatype: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Sh2ampConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Sh2ampParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the input image consisting of spherical harmonic (SH)\
            coefficients.
        directions: the list of directions along which the SH functions will be\
            sampled, generated using the dirgen command.
        output: the output image consisting of the amplitude of the SH\
            functions along the specified directions.
        nonnegative: cap all negative amplitudes to zero.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        strides: specify the strides of the output data in memory; either as a\
            comma-separated list of (signed) integers, or as a template image from\
            which the strides shall be extracted and used. The actual strides\
            produced will depend on whether the output image format can support it.
        datatype: specify output image data type. Valid choices are: float32,\
            float32le, float32be, float64, float64le, float64be, int64, uint64,\
            int64le, uint64le, int64be, uint64be, int32, uint32, int32le, uint32le,\
            int32be, uint32be, int16, uint16, int16le, uint16le, int16be, uint16be,\
            cfloat32, cfloat32le, cfloat32be, cfloat64, cfloat64le, cfloat64be,\
            int8, uint8, bit.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/sh2amp",
        "nonnegative": nonnegative,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
        "directions": directions,
        "output": output,
    }
    if grad is not None:
        params["grad"] = grad
    if fslgrad is not None:
        params["fslgrad"] = fslgrad
    if strides is not None:
        params["strides"] = strides
    if datatype is not None:
        params["datatype"] = datatype
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def sh2amp_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Sh2ampParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("nonnegative", False) is None:
        raise StyxValidationError("`nonnegative` must not be None")
    if not isinstance(params["nonnegative"], bool):
        raise StyxValidationError(f'`nonnegative` has the wrong type: Received `{type(params.get("nonnegative", False))}` expected `bool`')
    if params.get("grad", None) is not None:
        if not isinstance(params["grad"], (pathlib.Path, str)):
            raise StyxValidationError(f'`grad` has the wrong type: Received `{type(params.get("grad", None))}` expected `InputPathType | None`')
    if params.get("fslgrad", None) is not None:
        sh2amp_fslgrad_validate(params["fslgrad"])
    if params.get("strides", None) is not None:
        if not isinstance(params["strides"], dict):
            raise StyxValidationError(f'Params object has the wrong type \'{type(params["strides"])}\'')
        if "@type" not in params["strides"]:
            raise StyxValidationError("Params object is missing `@type`")
        if params["strides"]["@type"] not in ["VariousString", "VariousFile"]:
            raise StyxValidationError("Parameter `strides`s `@type` must be one of [\"VariousString\", \"VariousFile\"]")
        sh2amp_strides_validate_dyn_fn(params["strides"]["@type"])(params["strides"])
    if params.get("datatype", None) is not None:
        if not isinstance(params["datatype"], str):
            raise StyxValidationError(f'`datatype` has the wrong type: Received `{type(params.get("datatype", None))}` expected `str | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[Sh2ampConfigParamsDict] | None`')
        for e in params["config"]:
            sh2amp_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("directions", None) is None:
        raise StyxValidationError("`directions` must not be None")
    if not isinstance(params["directions"], (pathlib.Path, str)):
        raise StyxValidationError(f'`directions` has the wrong type: Received `{type(params.get("directions", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def sh2amp_cargs(
    params: Sh2ampParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("sh2amp")
    if params.get("nonnegative", False):
        cargs.append("-nonnegative")
    if params.get("grad", None) is not None:
        cargs.extend([
            "-grad",
            execution.input_file(params.get("grad", None))
        ])
    if params.get("fslgrad", None) is not None:
        cargs.extend(sh2amp_fslgrad_cargs(params.get("fslgrad", None), execution))
    if params.get("strides", None) is not None:
        cargs.extend([
            "-strides",
            *sh2amp_strides_cargs_dyn_fn(params.get("strides", None)["@type"])(params.get("strides", None), execution)
        ])
    if params.get("datatype", None) is not None:
        cargs.extend([
            "-datatype",
            params.get("datatype", None)
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [sh2amp_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(execution.input_file(params.get("directions", None)))
    cargs.append(params.get("output", None))
    return cargs


def sh2amp_outputs(
    params: Sh2ampParamsDict,
    execution: Execution,
) -> Sh2ampOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Sh2ampOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def sh2amp_execute(
    params: Sh2ampParamsDict,
    runner: Runner | None = None,
) -> Sh2ampOutputs:
    """
    sh2amp
    
    Evaluate the amplitude of an image of spherical harmonic functions along
    specified directions.
    
    The input image should consist of a 4D or 5D image, with SH coefficients
    along the 4th dimension according to the convention below. If 4D (or size 1
    along the 5th dimension), the program expects to be provided with a single
    shell of directions. If 5D, each set of coefficients along the 5th dimension
    is understood to correspond to a different shell.
    
    The directions can be provided as:
    - a 2-column ASCII text file contained azimuth / elevation pairs (as
    produced by dirgen)
    - a 3-column ASCII text file containing x, y, z Cartesian direction vectors
    (as produced by dirgen -cart)
    - a 4-column ASCII text file containing the x, y, z, b components of a full
    DW encoding scheme (in MRtrix format, see main documentation for details).
    - an image file whose header contains a valid DW encoding scheme
    
    If a full DW encoding is provided, the number of shells needs to match those
    found in the input image of coefficients (i.e. its size along the 5th
    dimension). If needed, the -shell option can be used to pick out the
    specific shell(s) of interest.
    
    If the input image contains multiple shells (its size along the 5th
    dimension is greater than one), the program will expect the direction set to
    contain multiple shells, which can only be provided as a full DW encodings
    (the last two options in the list above).
    
    The spherical harmonic coefficients are stored according the conventions
    described the main documentation, which can be found at the following link:
    https://mrtrix.readthedocs.io/en/3.0.4/concepts/spherical_harmonics.html
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Sh2ampOutputs`).
    """
    sh2amp_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SH2AMP_METADATA)
    params = execution.params(params)
    cargs = sh2amp_cargs(params, execution)
    ret = sh2amp_outputs(params, execution)
    execution.run(cargs)
    return ret


def sh2amp(
    input_: InputPathType,
    directions: InputPathType,
    output: str,
    nonnegative: bool = False,
    grad: InputPathType | None = None,
    fslgrad: Sh2ampFslgradParamsDict | None = None,
    strides: typing.Union[Sh2ampVariousStringParamsDictTagged, Sh2ampVariousFileParamsDictTagged] | None = None,
    datatype: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Sh2ampConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Sh2ampOutputs:
    """
    sh2amp
    
    Evaluate the amplitude of an image of spherical harmonic functions along
    specified directions.
    
    The input image should consist of a 4D or 5D image, with SH coefficients
    along the 4th dimension according to the convention below. If 4D (or size 1
    along the 5th dimension), the program expects to be provided with a single
    shell of directions. If 5D, each set of coefficients along the 5th dimension
    is understood to correspond to a different shell.
    
    The directions can be provided as:
    - a 2-column ASCII text file contained azimuth / elevation pairs (as
    produced by dirgen)
    - a 3-column ASCII text file containing x, y, z Cartesian direction vectors
    (as produced by dirgen -cart)
    - a 4-column ASCII text file containing the x, y, z, b components of a full
    DW encoding scheme (in MRtrix format, see main documentation for details).
    - an image file whose header contains a valid DW encoding scheme
    
    If a full DW encoding is provided, the number of shells needs to match those
    found in the input image of coefficients (i.e. its size along the 5th
    dimension). If needed, the -shell option can be used to pick out the
    specific shell(s) of interest.
    
    If the input image contains multiple shells (its size along the 5th
    dimension is greater than one), the program will expect the direction set to
    contain multiple shells, which can only be provided as a full DW encodings
    (the last two options in the list above).
    
    The spherical harmonic coefficients are stored according the conventions
    described the main documentation, which can be found at the following link:
    https://mrtrix.readthedocs.io/en/3.0.4/concepts/spherical_harmonics.html
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: the input image consisting of spherical harmonic (SH)\
            coefficients.
        directions: the list of directions along which the SH functions will be\
            sampled, generated using the dirgen command.
        output: the output image consisting of the amplitude of the SH\
            functions along the specified directions.
        nonnegative: cap all negative amplitudes to zero.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        strides: specify the strides of the output data in memory; either as a\
            comma-separated list of (signed) integers, or as a template image from\
            which the strides shall be extracted and used. The actual strides\
            produced will depend on whether the output image format can support it.
        datatype: specify output image data type. Valid choices are: float32,\
            float32le, float32be, float64, float64le, float64be, int64, uint64,\
            int64le, uint64le, int64be, uint64be, int32, uint32, int32le, uint32le,\
            int32be, uint32be, int16, uint16, int16le, uint16le, int16be, uint16be,\
            cfloat32, cfloat32le, cfloat32be, cfloat64, cfloat64le, cfloat64be,\
            int8, uint8, bit.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Sh2ampOutputs`).
    """
    params = sh2amp_params(
        nonnegative=nonnegative,
        grad=grad,
        fslgrad=fslgrad,
        strides=strides,
        datatype=datatype,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        directions=directions,
        output=output,
    )
    return sh2amp_execute(params, runner)


__all__ = [
    "SH2AMP_METADATA",
    "Sh2ampConfigParamsDict",
    "Sh2ampConfigParamsDictTagged",
    "Sh2ampFslgradParamsDict",
    "Sh2ampFslgradParamsDictTagged",
    "Sh2ampOutputs",
    "Sh2ampParamsDict",
    "Sh2ampParamsDictTagged",
    "Sh2ampVariousFileParamsDict",
    "Sh2ampVariousFileParamsDictTagged",
    "Sh2ampVariousStringParamsDict",
    "Sh2ampVariousStringParamsDictTagged",
    "sh2amp",
    "sh2amp_config",
    "sh2amp_execute",
    "sh2amp_fslgrad",
    "sh2amp_params",
    "sh2amp_various_file",
    "sh2amp_various_string",
]
